@@ -1,6 +1,6 @@
 from PyQt6.QtCore import Qt, QObject, QEvent, QSize
 from PyQt6.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout, QLabel, QPushButton, QLineEdit, QComboBox, QDialog, QToolButton, QWidgetAction, QCheckBox, QTableWidget, QHeaderView
-from PyQt6.QtGui import QIcon
+from PyQt6.QtGui import QIcon, QFont, QFontMetrics
 import re
 import requests
 import importlib.resources
@@ -27,10 +27,10 @@ def create_form(
     title_font_color = "#ffffff",
     title_background_color = "#1e1e1e",
     title_padding = 15,
+    title_padding_left = None,
     title_padding_top = None,
     title_padding_right = None,
     title_padding_bottom = None,
-    title_padding_left = None,
     title_border_width = 0,
     title_border_color = "#ffffff",
     title_border_radius = 0
@@ -44,10 +44,10 @@ def create_form(
         font_color = title_font_color,
         background_color = title_background_color,
         padding = title_padding,
+        padding_left = title_padding_left,
         padding_top = title_padding_top,
         padding_right = title_padding_right,
         padding_bottom = title_padding_bottom,
-        padding_left = title_padding_left,
         border_width = title_border_width,
         border_color = title_border_color,
         border_radius = title_border_radius
@@ -85,6 +85,46 @@ def create_form(
     body_layout.setStretch(1, 4)
     return main_layout
 
+def create_header(
+    title,
+    margin_left = 25,
+    margin_top = 25,
+    margin_right = 25,
+    margin_bottom = 25,
+    title_font_family = "Segoe UI",
+    title_font_size = 36,
+    title_font_color = "#ffffff",
+    title_background_color = "#1e1e1e",
+    title_padding = 15,
+    title_padding_left = None,
+    title_padding_top = None,
+    title_padding_right = None,
+    title_padding_bottom = None,
+    title_border_width = 0,
+    title_border_color = "#ffffff",
+    title_border_radius = 0
+):
+    main_layout = QVBoxLayout()
+    main_layout.setContentsMargins(margin_left, margin_top, margin_right, margin_bottom)
+    title_layout = create_title(
+        text = title,
+        font_family = title_font_family,
+        font_size = title_font_size,
+        font_color = title_font_color,
+        background_color = title_background_color,
+        padding = title_padding,
+        padding_left = title_padding_left,
+        padding_top = title_padding_top,
+        padding_right = title_padding_right,
+        padding_bottom = title_padding_bottom,
+        border_width = title_border_width,
+        border_color = title_border_color,
+        border_radius = title_border_radius
+    )
+    main_layout.addLayout(title_layout)
+    main_layout.addStretch()
+    return main_layout
+
 def create_title(
     text,
     font_family = "Segoe UI",
@@ -92,10 +132,10 @@ def create_title(
     font_color = "#ffffff",
     background_color = "#1e1e1e",
     padding = 15,
+    padding_left = None,
     padding_top = None,
     padding_right = None,
     padding_bottom = None,
-    padding_left = None,
     border_width = 0,
     border_color = "#ffffff",
     border_radius = 0
@@ -111,10 +151,10 @@ def create_title(
         font_weight = "bold",
         background_color = background_color,
         padding = padding,
+        padding_left = padding_left,
         padding_top = padding_top,
         padding_right = padding_right,
         padding_bottom = padding_bottom,
-        padding_left = padding_left,
         border_width = border_width,
         border_color = border_color,
         border_radius = border_radius
@@ -130,29 +170,29 @@ def create_label(
         font_weight = "normal",
         background_color = "#1e1e1e",
         padding = 15,
+        padding_left = None,
         padding_top = None,
         padding_right = None,
         padding_bottom = None,
-        padding_left = None,
         border_width = 0,
         border_color = "#ffffff",
         border_radius = 0
 ):
     label = QLabel(text)
+    padding_left_value = padding_left if padding_left is not None else padding
     padding_top_value = padding_top if padding_top is not None else padding
     padding_right_value = padding_right if padding_right is not None else padding
     padding_bottom_value = padding_bottom if padding_bottom is not None else padding
-    padding_left_value = padding_left if padding_left is not None else padding
     style = f"""
         font-family: {font_family};
         font-size: {font_size}px;
         color: {font_color};
         font-weight: {font_weight};
         background-color: {background_color};
+        padding-left: {padding_left_value}px;
         padding-top: {padding_top_value}px;
         padding-right: {padding_right_value}px;
         padding-bottom: {padding_bottom_value}px;
-        padding-left: {padding_left_value}px;
         border: {border_width}px solid {border_color};
         border-radius: {border_radius}px;
     """
@@ -167,10 +207,10 @@ def create_button(
     font_weight = "bold",
     background_color = "#1e1e1e",
     padding = 15,
+    padding_left = None,
     padding_top = None,
     padding_right = None,
     padding_bottom = None,
-    padding_left = None,
     border_width = 2,
     border_color = "#5c5c5c",
     border_radius = 0,
@@ -185,11 +225,22 @@ def create_button(
     disabled_border_width = 2,
     disabled_border_color = "#4a4a4a"
 ):
-    button = QPushButton(text)
+    button = QPushButton()
+    button.setMaximumWidth(360)
+    padding_left_value = padding_left if padding_left is not None else padding
     padding_top_value = padding_top if padding_top is not None else padding
     padding_right_value = padding_right if padding_right is not None else padding
     padding_bottom_value = padding_bottom if padding_bottom is not None else padding
-    padding_left_value = padding_left if padding_left is not None else padding
+    max_text_width = 360 - padding_left_value - padding_right_value - 2 * border_width
+    font = QFont(font_family, font_size)
+    if font_weight == "bold": font.setWeight(QFont.Weight.Bold)
+    else: font.setWeight(QFont.Weight.Normal)
+    metrics = QFontMetrics(font)
+    text_width = metrics.horizontalAdvance(text)
+    if text_width > max_text_width:
+        html_text = f"<div style = \"white-space: normal; text-align: center;\">{text}</div>"
+        button.setText(html_text)
+    else: button.setText(text)
     style_sheet = f"""
         QPushButton {{
             font-family: {font_family};
@@ -197,10 +248,10 @@ def create_button(
             color: {font_color};
             font-weight: {font_weight};
             background-color: {background_color};
+            padding-left: {padding_left_value}px;
             padding-top: {padding_top_value}px;
             padding-right: {padding_right_value}px;
             padding-bottom: {padding_bottom_value}px;
-            padding-left: {padding_left_value}px;
             border: {border_width}px solid {border_color};
             border-radius: {border_radius}px;
         }}
@@ -229,10 +280,10 @@ def create_text_box(
     font_weight = "normal",
     background_color = "#1e1e1e",
     padding = 15,
+    padding_left = None,
     padding_top = None,
     padding_right = None,
     padding_bottom = None,
-    padding_left = None,
     border_width = 2,
     border_color = "#5c5c5c",
     border_radius = 0,
@@ -254,10 +305,10 @@ def create_text_box(
 ):
     text_box = QLineEdit()
     text_box.setPlaceholderText(placeholder_text)
+    padding_left_value = padding_left if padding_left is not None else padding
     padding_top_value = padding_top if padding_top is not None else padding
     padding_right_value = padding_right if padding_right is not None else padding
     padding_bottom_value = padding_bottom if padding_bottom is not None else padding
-    padding_left_value = padding_left if padding_left is not None else padding
     style_sheet = f"""
         QLineEdit {{
             font-family: {font_family};
@@ -265,10 +316,10 @@ def create_text_box(
             color: {font_color};
             font-weight: {font_weight};
             background-color: {background_color};
+            padding-left: {padding_left_value}px;
             padding-top: {padding_top_value}px;
             padding-right: {padding_right_value}px;
             padding-bottom: {padding_bottom_value}px;
-            padding-left: {padding_left_value}px;
             border: {border_width}px solid {border_color};
             border-radius: {border_radius}px;
         }}
@@ -343,10 +394,10 @@ def create_combo_box(
     font_color = "#ffffff",
     background_color = "#1e1e1e",
     padding = 15,
+    padding_left = None,
     padding_top = None,
     padding_right = None,
     padding_bottom = None,
-    padding_left = None,
     border_width = 2,
     border_color = "#5c5c5c",
     border_radius = 0,
@@ -368,10 +419,10 @@ def create_combo_box(
     combo_box.addItems(items)
     combo_box.setCurrentIndex(-1)
     
+    padding_left_value = padding_left if padding_left is not None else padding
     padding_top_value = padding_top if padding_top is not None else padding
     padding_right_value = padding_right if padding_right is not None else padding
     padding_bottom_value = padding_bottom if padding_bottom is not None else padding
-    padding_left_value = padding_left if padding_left is not None else padding
 
     def get_stylesheet(font_color):
         return f"""
@@ -380,10 +431,10 @@ def create_combo_box(
                 font-size: {font_size}px;
                 color: {font_color};
                 background-color: {background_color};
+                padding-left: {padding_left_value}px;
                 padding-top: {padding_top_value}px;
                 padding-right: {padding_right_value}px;
                 padding-bottom: {padding_bottom_value}px;
-                padding-left: {padding_left_value}px;
                 border: {border_width}px solid {border_color};
                 border-radius: {border_radius}px;
             }}
@@ -422,10 +473,10 @@ def create_checkbox(
     font_color = "#ffffff",
     background_color = "#1e1e1e",
     padding = 5,
+    padding_left = None,
     padding_top = None,
     padding_right = None,
     padding_bottom = None,
-    padding_left = None,
     indicator_background_color = "#1e1e1e",
     indicator_border_width = 1,
     indicator_border_color = "#5c5c5c",
@@ -445,20 +496,20 @@ def create_checkbox(
     disabled_border_color = "#4a4a4a"
 ):
     check_box = QCheckBox(text)
+    padding_left_value = padding_left if padding_left is not None else padding
     padding_top_value = padding_top if padding_top is not None else padding
     padding_right_value = padding_right if padding_right is not None else padding
     padding_bottom_value = padding_bottom if padding_bottom is not None else padding
-    padding_left_value = padding_left if padding_left is not None else padding
     style_sheet = f"""
         QCheckBox {{
             font-family: {font_family};
             font-size: {font_size}px;
             color: {font_color};
             background-color: {background_color};
+            padding-left: {padding_left_value}px;
             padding-top: {padding_top_value}px;
             padding-right: {padding_right_value}px;
             padding-bottom: {padding_bottom_value}px;
-            padding-left: {padding_left_value}px;
         }}
         QCheckBox::indicator {{
             image: none;
@@ -487,16 +538,16 @@ def create_checkbox(
             background-color: {disabled_background_color};
             border: {disabled_border_width}px solid {disabled_border_color};
         }}
-    """
+    """ 
     check_box.setStyleSheet(style_sheet)
     return check_box
 
 def create_information_message_box(
     text,
-    top_margin = 25,
-    bottom_margin = 25,
     left_margin = 25,
+    top_margin = 25,
     right_margin = 25,
+    bottom_margin = 25,
     spacing = 10,
     background_color = "#1e1e1e",
     border_width = 2,
@@ -506,10 +557,10 @@ def create_information_message_box(
     label_font_size = 14,
     label_font_color = "#ffffff",
     label_padding = 15,
+    label_padding_left = None,
     label_padding_top = None,
     label_padding_right = None,
     label_padding_bottom = None,
-    label_padding_left = None,
     label_border_width = 0,
     label_border_color = "#ffffff",
     label_border_radius = 0,
@@ -519,10 +570,10 @@ def create_information_message_box(
     button_font_color = "#ffffff",
     button_background_color = "#1e1e1e",
     button_padding = 15,
+    button_padding_left = None,
     button_padding_top = None,
     button_padding_right = None,
     button_padding_bottom = None,
-    button_padding_left = None,
     button_border_width = 2,
     button_border_color = "#5c5c5c",
     button_border_radius = 0,
@@ -560,10 +611,10 @@ def create_information_message_box(
         font_color=  label_font_color,
         background_color = "transparent",
         padding = label_padding,
+        padding_left = label_padding_left,
         padding_top = label_padding_top,
         padding_right = label_padding_right,
         padding_bottom = label_padding_bottom,
-        padding_left = label_padding_left,
         border_width = label_border_width,
         border_color = label_border_color,
         border_radius = label_border_radius
@@ -578,10 +629,10 @@ def create_information_message_box(
         font_color = button_font_color,
         background_color = button_background_color,
         padding = button_padding,
+        padding_left = button_padding_left,
         padding_top = button_padding_top,
         padding_right = button_padding_right,
         padding_bottom = button_padding_bottom,
-        padding_left = button_padding_left,
         border_width = button_border_width,
         border_color = button_border_color,
         border_radius = button_border_radius,
@@ -602,10 +653,10 @@ def create_information_message_box(
     
 def create_confirmation_message_box(
     text,
-    top_margin = 25,
-    bottom_margin = 25,
     left_margin = 25,
+    top_margin = 25,
     right_margin = 25,
+    bottom_margin = 25,
     main_spacing = 10,
     button_spacing = 10,
     background_color = "#1e1e1e",
@@ -616,10 +667,10 @@ def create_confirmation_message_box(
     label_font_size = 14,
     label_font_color = "#ffffff",
     label_padding = 15,
+    label_padding_left = None,
     label_padding_top = None,
     label_padding_right = None,
     label_padding_bottom = None,
-    label_padding_left = None,
     label_border_width = 0,
     label_border_color = "#ffffff",
     label_border_radius = 0,
@@ -628,10 +679,10 @@ def create_confirmation_message_box(
     button_font_color = "#ffffff",
     button_background_color = "#1e1e1e",
     button_padding = 15,
+    button_padding_left = None,
     button_padding_top = None,
     button_padding_right = None,
     button_padding_bottom = None,
-    button_padding_left = None,
     button_border_width = 2,
     button_border_color = "#5c5c5c",
     button_border_radius = 0,
@@ -669,10 +720,10 @@ def create_confirmation_message_box(
         font_color = label_font_color,
         background_color = "transparent",
         padding = label_padding,
+        padding_left = label_padding_left,
         padding_top = label_padding_top,
         padding_right = label_padding_right,
         padding_bottom = label_padding_bottom,
-        padding_left = label_padding_left,
         border_width = label_border_width,
         border_color = label_border_color,
         border_radius = label_border_radius
@@ -690,10 +741,10 @@ def create_confirmation_message_box(
         font_color = button_font_color,
         background_color = button_background_color,
         padding = button_padding,
+        padding_left = button_padding_left,
         padding_top = button_padding_top,
         padding_right = button_padding_right,
         padding_bottom = button_padding_bottom,
-        padding_left = button_padding_left,
         border_width = button_border_width,
         border_color = button_border_color,
         border_radius = button_border_radius,
@@ -717,10 +768,10 @@ def create_confirmation_message_box(
         font_color = button_font_color,
         background_color = button_background_color,
         padding = button_padding,
+        padding_left = button_padding_left,
         padding_top = button_padding_top,
         padding_right = button_padding_right,
         padding_bottom = button_padding_bottom,
-        padding_left = button_padding_left,
         border_width = button_border_width,
         border_color = button_border_color,
         border_radius = button_border_radius,
